/*********************************************************************
 * Desktop Station AUTO Sketch for DS shield series
 *
 * Copyright (C) 2017 Yaasan
 *
 */

// include the library code:
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "Functions.h"
#include "TrackReporterS88_DS.h"

#define MAX_S88DECODER 1

#define LEDSTATE_STOP 0
#define LEDSTATE_RUN 1
#define LEDSTATE_ERR 2

#define PIN_RUNLED	2

#define MODE_SPEEDSTEP128 2

#define THRESHOLD_EDC 276  // 1024d * (1.2 / 8) * 9V / 5V
#define THRESHOLD_OV  614  // 1024d * (1.2 / 8) * 20V / 5V


/* Decralation classes */
DSCoreLib DSCore;
TrackReporterS88_DS reporter(MAX_S88DECODER);

/* LED functions */
byte gLED_State = LEDSTATE_STOP;
byte gLED_Toggle = 0;

//Task Schedule
unsigned long gPreviousL8 = 0; // 1000ms interval


//コマンド関数定義
void CMD_Wait(word inWaitMs);
void CMD_Power(byte inState);
void CMD_LocSpeed(word inAddress, word inSpeed);
void CMD_LocDirection(word inAddress, byte inDirection);
void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState);
void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo);
void CMD_Turnout(word inAddress, byte inDirection);
void CMD_ScanS88(void);
byte CMD_GetS88(byte inNo);
void CMD_END(void);


//CheapContriller用 190908 fujigaya2
#define SPEED_REF A2
#define BTN_DIR A3
#define BTN_F0 6
#define BTN_F1 7
#define BTN_F2 8
#define BTN_F3 11
#define BTN_F4 12




/***********************************************************************
* 
* Boot setup
* 
************************************************************************/

void setup()
{

	cli();
	
	//タイマ0を強制的に戻すr
	TCCR0B = (TCCR0B & 0b11111000) | 0x03;
	//Timer1 init
	TCCR1A = 0;
	TCCR1B = 0;
	TCNT1  = 0;

	OCR1A = 20;            // 1Hz = 62500d, 100Hz = 625d
	TCCR1B |= (1 << WGM12);   // CTC mode
	TCCR1B |= (1 << CS12);    // 256 prescaler 
	TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt

	DSCore.Init();
  
  //タイマ0を強制的に戻すr
  TCCR0B = (TCCR0B & 0b11111000) | 0x03;
	
	//Pin mode
	pinMode(A1, INPUT);//Edc
	pinMode(PIN_RUNLED, OUTPUT);
	
	/* LED turn on */
	changeLED(0);
	
	/* Clear data of the S88 decoder */
	reporter.refresh();
	
	/* Debug message for PC */
	Serial.begin(115200);
	Serial.println("--------------------------------------");
	Serial.println("Desktop Station AUTO R1d 2017/11/12  ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");
	
	//Reset task
	gPreviousL8 = millis();
	
	
	
	sei();    //ENABLE INTERRUPTION
	
	/* 線路に電源遮断 */
	DS_Power(0);
	
	//User initial
	user_init();
	
}

/***********************************************************************
* 
* Command for locomotives and turnouts
* 
************************************************************************/

void CMD_Wait(word inWaitMs)
{
	unsigned long aTemp = millis() + (inWaitMs >> 2);
	
	//なんか時間が合わない。Timer0のプリスケーラがおかしい？
	while ( aTemp > millis() )
	{
		//NOP
		__asm__("nop\n\t");
	}
	
}

void CMD_Power(byte inState)
{
	/* LED状態の変更 */
	changeLED(inState);
	
	DSCore.SetPower(inState);
}

void CMD_LocSpeed(word inAddress, word inSpeed)
{
	byte aMode = MODE_SPEEDSTEP128;
	
	if( inAddress <= 256)
	{
		aMode = 0;
	}
	
	
	DSCore.SetLocoSpeedEx(inAddress, inSpeed, aMode);
}

void CMD_LocDirection(word inAddress, byte inDirection)
{
	DSCore.SetLocoDirection(inAddress, inDirection);
}

void CMD_LocFunction(word inAddress, byte inFunctionNo, byte inFunctionState)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, inFunctionState);
}

void CMD_LocFunctionMomentary(word inAddress, byte inFunctionNo)
{
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 1);
	CMD_Wait(200);
	DSCore.SetLocoFunction(inAddress, inFunctionNo + 1, 0);
}


void CMD_Turnout(word inAddress, byte inDirection)
{
	DSCore.SetTurnout(inAddress - 1, inDirection);
}

void CMD_ScanS88(void)
{
	reporter.refresh(MAX_S88DECODER);
}

byte CMD_GetS88(byte inNo)
{
	word aFlags = reporter.getByte(0) + reporter.getByte(1);
	
	return (aFlags >> inNo) & 1;
}

void CMD_END(void)
{
	/* 永久ループ（終了） */
	while(1){}
}


/***********************************************************************
* 
* Timer Interval
* 
************************************************************************/

ISR(TIMER1_COMPA_vect)
{
	uint8_t aErrNo = 0;
	word aEdc;
	aErrNo = 0;
	
	//パルスジェネレータのスキャン
	DSCore.Scan();
	
	if( (millis() - gPreviousL8) >= 200)
	{
		//Reset task
		gPreviousL8 = millis();
		
		//LED Control
		taskLED();
		
		/* Sensor check */
		aEdc = analogRead(A1);

		/* OV check */
		if( aEdc > THRESHOLD_OV )
		{
			aErrNo = 2;
		}
		/* LV check */
		if( aEdc < THRESHOLD_EDC )
		{
			aErrNo = 1;
			
		}
		/* エラー処理 */
    //CheapContriller用 190908 fujigaya2
		//エラー処理なし（回路入れていないため）
		//if( aErrNo > 0)
		//{
		//	if( DSCore.IsPower() == 1)
		//	{
		//		PowerOffByErr(aErrNo);
		//	}
			
		//}
	}
	
}

/***********************************************************************
* 
* Main Loop
* 
************************************************************************/

void loop()
{
	
	user_program();
	
}

/***********************************************************************
* 
* RUN LED Control functions
* 
************************************************************************/

void PowerOffByErr(uint8_t inErrNo)
{
	DSCore.SetPower(0);
	
	/* 状態変更 */
	gLED_State = LEDSTATE_ERR;
}


void changeLED(byte inState)
{
	if( inState == 1)
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_RUN;
	}
	else
	{
		/* 状態変更 */
		gLED_State = LEDSTATE_STOP;
	}
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		digitalWrite(PIN_RUNLED, 0);
		
		break;
		
	case LEDSTATE_RUN:
		digitalWrite(PIN_RUNLED, 1);
		break;
		
	case LEDSTATE_ERR:
		break;
	}
}

void taskLED(void)
{
	
	
	switch( gLED_State)
	{
	case LEDSTATE_STOP:
		break;
		
	case LEDSTATE_RUN:
		break;
		
	case LEDSTATE_ERR:
		digitalWrite(PIN_RUNLED, gLED_Toggle);
		gLED_Toggle = (~gLED_Toggle) & 1;
		
		break;
	}
}

/*****************************************************************/
/*  User Program Area                                            */
/*****************************************************************/

void user_init(void)
{
  //CheapContriller用 190908 fujigaya2
  
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(BTN_F0, INPUT_PULLUP);
  pinMode(BTN_F1, INPUT_PULLUP);
  pinMode(BTN_F2, INPUT_PULLUP);
  pinMode(BTN_F3, INPUT_PULLUP);
  pinMode(BTN_F4, INPUT_PULLUP);
  pinMode(BTN_DIR, INPUT_PULLUP);    	
	
	//SPEED 0
	//CMD_LocSpeed(ADDR_MM2 + 2, 0);
	CMD_LocSpeed(ADDR_DCC + 3, 0);
  
	//POWER ON
	CMD_Power(1);
	Serial.println("Power On");
}


void user_program(void)
{
  //CheapContriller用 190908 fujigaya2
  //ボタンが押されたとき、digitalReadは0を返す。押さないときは1を返す。

  static boolean chattering_flag = false;
  static unsigned long gPreviousL1 = 0;
  static unsigned long gPreviousL2 = 0;

  static boolean state_btn_f0 = false;
  static boolean state_btn_f1 = false;
  static boolean state_btn_f2 = false;
  static boolean state_btn_f3 = false;
  static boolean state_btn_f4 = false;
  static boolean state_btn_dir = false; //false = for,true =rev



  //チャタリング回避　200ms
  if(  chattering_flag == true)
  {
    if( (millis() - gPreviousL1) >= 200 ) 
    {
      chattering_flag = false;
    }
    else
    {
      return;
    }
  }
  
  if( digitalRead(BTN_F0) == 0)
  {
    state_btn_f0 = ! state_btn_f0;
    CMD_LocFunction(ADDR_DCC + 3, 0, int(state_btn_f0));
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }
  if( digitalRead(BTN_F1) == 0)
  {
    state_btn_f1 = ! state_btn_f1;
    CMD_Wait(100);    
    CMD_LocFunction(ADDR_DCC + 3, 1, int(state_btn_f1));
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }
  if( digitalRead(BTN_F2) == 0)
  {
    state_btn_f2 = ! state_btn_f2;
    CMD_Wait(100);    
    CMD_LocFunction(ADDR_DCC + 3, 2, int(state_btn_f2));
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }
  if( digitalRead(BTN_F3) == 0)
  {
    state_btn_f3 = ! state_btn_f3;
    CMD_Wait(100);    
    CMD_LocFunction(ADDR_DCC + 3, 3, int(state_btn_f3));
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }
  if( digitalRead(BTN_F4) == 0)
  {
    state_btn_f4 = ! state_btn_f4;
    CMD_Wait(100);    
    CMD_LocFunction(ADDR_DCC + 3, 4, int(state_btn_f4));
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }
  if( digitalRead(BTN_DIR) == 0)
  {
    state_btn_dir = ! state_btn_dir;
    CMD_Wait(100);    
    CMD_LocDirection(ADDR_DCC + 3, int(state_btn_dir) * 2);
    chattering_flag = true;
    gPreviousL1 = millis();
    return;
  }

  if( (millis() - gPreviousL2) >= 200)
  {
    int current_speed_volume = analogRead(SPEED_REF);
    //0-1023
    //Serial.println(current_speed_volume);
    CMD_LocSpeed(ADDR_DCC + 3, current_speed_volume);
    gPreviousL2 =  millis();
    return;
  }


}
